#!/usr/local/bin/perl5.6.0 -w

# External Modules

# Constraint variables declaration before using them
use strict;
##########################################

use Data::Dumper;

eval "require Bio::PSU::Utils::EstwiseParsing";
# use Bio::PSU::Utils::EstwiseParsing;

# eval "require Bio::PSU::Utils::EstwiseHit";
# use Bio::PSU::Utils::EstwiseHit;

##########################################
=pod

=item * Configuration loading from the command line

=cut
##########################################

my $_help           = "-help";
my $_est            = "-est";
my $_pfam           = "-pfam";
my $_sequences_url  = "-urlfasta";

# no ESTs input per default
my $ests         = "-1";
my $pfam         = "/nfs/disk100/pubseq/Pfam/RELEASES/CURRENT/Pfam_ls";
my $pfam_mapping_file = "/nfs/pathsoft/external/bio-soft/proteinMappingFiles/pfamName2Accession.mapping";
my $ipro_mapping_file = "/nfs/pathsoft/external/bio-soft/proteinMappingFiles/protein2ipr.dat";
my $go_mapping_file   = "/nfs/pathsoft/external/bio-soft/proteinMappingFiles/interpro2go.txt";
my $sequences_url = "ftp://ftp.sanger.ac.uk/pub/pathogens/Glossina/GLOS.fasta.27.1.02";
my $_pfam_url = "http://www.sanger.ac.uk/cgi-bin/Pfam/getacc?";
my $_ipro_url = "http://www.ebi.ac.uk/interpro/IEntry?ac=";
my $_go_url = "http://golgi.ebi.ac.uk/ego/QuickGO?mode=display&query=";
my $_go_url_2 = "&entry=";

my $cur_dir = qx/pwd/;
chomp ($cur_dir);

# Check if in the command line there is an input file name (-i option)
# the other options are not compulsory (set of options per default)
my $lgth = @ARGV + 0;
# if no args, display the help message
if ($lgth == 0) {
  print "specify at least an ESTs input file with the option \"-est\"\n";
  display_help ();
  exit 1;
}
elsif ($lgth >= 1) {
  my $i = 0;
  foreach my $element (@ARGV) {
    if ($element eq $_help) {
      display_help ();
      exit 1;
    }
    elsif ($element eq $_est) {
      $ests = $ARGV[$i+1];
    }
    elsif ($element eq $_pfam) {
      $pfam = $ARGV[$i+1];
    }
    elsif ($element eq $_sequences_url) {
      $sequences_url = $ARGV[$i+1];
    }
    # if at least one of the options is not recognized, display the help message and exit
    elsif ($element =~ /^-/) {
      print STDERR "option not recognized: $element\n";
      display_help ();
      exit 1;
    }
    $i++;
  }
}
if ($ests eq "-1") {
  print STDERR "ERROR - specify at least the name of an ESTs input file with the -est option\n";
  print STDERR "Syntax :  ESTs_annotation.pl -est [ESTs file]\n";
  exit 1;
}

#########
## run estwisedb

print STDERR "running estwisedb ...\n";
my $output_file_name = run_estwisedb ($ests, $pfam);


#########
## Parse estwisedb file output

my $estwise_parser = Bio::PSU::Utils::EstwiseParsing->new (
							   estwiseOutput => $output_file_name,
							   pfamMapping => $pfam_mapping_file,
							   iproMapping => $ipro_mapping_file,
							   goMapping   => $go_mapping_file,
							  );

print STDERR "parsing object: " . Dumper ($estwise_parser) . "\n";

print STDERR "parsing estwisedb output...\n";
my @results = $estwise_parser->parse;


#########
## Generate HTML output

my $html_output_name = "estwisedb.$ests.html";
generateHTMLOutput ($html_output_name, @results);


#########
## end

# display the help message

sub display_help {
  print "ESTs_annotation.pl -help -est [ESTs File] -pfam [Pfam Library] -urlfasta [ESTs file URL]\n\n";
  print "\t-help a help message\n";
  print "\t-est the ESTs DB input - (no default, always specify it)\n";
  print "\t-pfam the Pfam library file - (default: ~pubseq/Pfam/RELEASES/CURRENT/Pfam_ls)\n";
  print "\t-urlfasta the sequences URL - (default is Glossina ESTs ftp URL)\n\n";
}

sub run_estwisedb {
  my ($ests, $pfam) = @_;
  my $output_file_name = "estwisedb.$ests.res";
  
  my @args = ("estwisedb -alg 333 -sum -pfam $pfam $ests > $output_file_name");
  system (@args);

  return $output_file_name;
}


sub generateHTMLOutput {
  my ($html_output_name, @results) = @_;

  print STDERR "generating html output ...\n";
  # print STDERR "dumping results, " . Dumper (@results) . "\n";

  open HTML, "> $html_output_name" or die "can't open file, $!\n";

  print HTML "<html><head></head><body>\n<h3>ESTs Annotations</h3>\n";
  print HTML "\n<a href=\"$sequences_url\">ESTs sequences</a>\n\n";
  print HTML "<table border><tr><th>EST</th><th>Strand</th><th>Pfam Name</th><th>Bits</th><th>IPRO</th><th>GO</th></tr>\n";

  foreach my $estwise_hit (@results) {
    print HTML "<tr>\n";

    my $est_acc   = $estwise_hit->getESTAccession;
    my $strand    = $estwise_hit->getStrand;
    my $pfam_name = $estwise_hit->getPfamName;
    my $pfam_acc  = $estwise_hit->getPfamAccession;
    my $bits      = $estwise_hit->getBits;
    my $ipro_acc  = $estwise_hit->getInterproAccession;
    my @go_accs   = $estwise_hit->getGOAccessions;

    print HTML "<td>$est_acc</td>\n";
    print HTML "<td align=\"center\">$strand</td>\n";
    
    if (not $pfam_acc =~ /undefined/i) {
      print HTML "<td><a href=\"$_pfam_url$pfam_acc\">$pfam_name</a></td>\n";
    }
    else {
      print HTML "<td>$pfam_name</td>\n";
    }
    print HTML "<td>$bits</td>\n";
    if (not $ipro_acc =~ /undefined/i) {
      print HTML "<td><a href=\"$_ipro_url$ipro_acc\">$ipro_acc</a></td>\n";
    }
    else {
      print HTML "<td>$ipro_acc</td>\n";
    }
 
    my $go_acc = pop (@go_accs);
    if (not $go_acc =~ /undefined/i) {  
      print HTML "<td><a href=\"$_go_url$go_acc$_go_url_2$go_acc\">$go_acc</a>";
    }
    else {
      print HTML "<td>$go_acc</td>\n"; 
    }
    
    while (my $go_acc = pop (@go_accs)) {
      print HTML ", <a href=\"$_go_url$go_acc$_go_url_2$go_acc\">$go_acc</a>";
    }
    print HTML "</td></tr>\n";
  }

  print HTML "</table>\n</body></html>\n";

  close HTML;
}
