#!/usr/local/bin/perl5.6.0 -w

=head1 NAME

blastwrap - Simple Blast search wrapper for aliasing databases

=head1 SYNOPSIS

 blastwrap -p

 blastwrap blastn %emnew queryseq.nt

 blastwrap wublastp %swir queryseq.aa

=head1 DESCRIPTION

This script is a wrapper for blast searches allowing a single database
alias for split databases. To use it precede your normal Blast command
(any version of Blast) with 'blastwrap' and substitute the database
name for an alias with a leading '%' e.g. %embl_other, %embl_htg,
%swissprot.

To view the current aliases and database path use 'blastwrap -p'. The
script checks that the database is present and readable before
commencing the search. If it is not, or if the Blast program exits
with an error, the script will terminate.

The script will treat commands starting with 'blastall' as if they are
using NCBI Blast 2 and will generate a command line which searches a
virtual database consisting of all the parts combined and gives a
single result. NCBI Blast 1.4 and WU Blast do not support this
features so blastwrap simply iterates over the parts and performs a
search on each.

=head1 METHODS

None

=head1 AUTHOR

Keith James (kdj@sanger.ac.uk)

=head1 COPYRIGHT

Copyright (C) 2001 Genome Research Limited. All Rights Reserved.

=head1 DISCLAIMER

This software is provided "as is" without warranty of any kind. It may
be used, redistributed and/or modified under the same conditions as
Perl itself.

=cut

use strict;
use Getopt::Std;

my $dbpath = "/nfs/disk100/pubseq/blastdb";

my %dbhash = (dbest         => [map { "dbEST-$_" } (1..8)],
              dbgss         => [map { "dbGSS-$_" } (1..3)],
              dbsts         => ["dbSTS-1"],
              ecoli         => ["ecoli"],
              embl_htg      => [map { "embl_htg-$_"   } (1..7)],
              embl_other    => [map { "embl_other-$_" } (1..6)],
              embl_vertrna  => ["embl_vertrna-1"],
              emnew_htg     => [map{ "emnew_htg-$_" } (1..2)],
              emnew_other   => ["emnew_other-1"],
              emnew_vertrna => ["emnew_vertrna-1"],
              ensembl       => ["ensembl"],
              fugu          => ["fugu"],
              gbrna         => ["gbrna"],
              sptrembl      => ["sptrembl"],
              swir          => ["swir"],
              swall         => [map { "swall-$_" } (1..2)],
              swissprot     => ["swissprot"],
              swnew         => ["swnew"],
              trnew         => ["trnew"],
              vectors       => ["vectors"],
              yeast         => ["yeast"] );

my (%opts, $db_alias, $verbose);
my $usage = "Usage: blastwrap [-pv] <blast command>

\t-p print out the known databases and aliases
\t-v verbose mode; prints command lines executed to STDERR
";

getopts('hpv', \%opts);

defined $opts{h} and die $usage;
defined $opts{p} and print_db($dbpath, \%dbhash);
defined $opts{v} and $verbose = 1;

# this is passed to the system() call before calling the blast
# executable to maximise the memory available for blast
my $magic_ulimit_string = "ulimit -d unlimited;";

# try it and if it fails don't do it later
if (system "$magic_ulimit_string") {
  print STDERR "ulimit failed - continuing anyway\n";
  $magic_ulimit_string = "";
}

unless (defined $verbose) { $verbose = 0 }

my $command = join(" ", @ARGV);

if ($command =~ / %(\S+)/)
{
    $db_alias = $1
}
else
{
    print STDERR "Executing: $command\n" if $verbose;

    my $return = 0xffff & system "$magic_ulimit_string $command";
    if ($return)
    {
        trap($dbpath, "user database", $command,
             "search exited with return code: $return")
    }
    exit;
}

foreach my $db (@{$dbhash{$db_alias}})
{
    check_db($dbpath, $db, $command);
}

my @commands;

# Hack to support NCBI blast 1.4/WU Blast and NCBI Blast 2
if ($command =~ /^blastall/)
{
    my $db_list = join(" ", @{$dbhash{$db_alias}});

    $command =~ s/%\S+/\"$db_list\"/;
    push(@commands, $command);
}
else
{
    foreach my $db (@{$dbhash{$db_alias}})
    {
        # make a copy of the command line
        my $command_template = $command;

        # Subsitute a database filename for the placeholder
        $command_template =~ s/%\S+/$db/;
        push(@commands, $command_template);
    }
}

# Run each generated command line. There will be only 1 for NCBI Blast
# 2, but >= 1 for NCBI blast 1.4/WU Blast
foreach my $command (@commands)
{
    print STDERR "Executing: $command\n" if $verbose;

    my $return = 0xffff & system "$magic_ulimit_string $command";
    if ($return)
    {
        trap($db_alias, $command, "search exited with return code: $return")
    }
}

sub check_db
{
    my ($dbpath, $db, $command) = @_;

    unless (-e "$dbpath/$db")
    {
        trap($db, $command, "database not found")
    }

    unless (-r "$dbpath/$db")
    {
        trap($db, $command, "database not readable")
    }
}

sub print_db
{
    my ($dbpath, $dbhash) = @_;

    print "Databases are located in:\n\t$dbpath\n";

    foreach my $db_alias (sort keys %$dbhash)
    {
        print "Alias $db_alias will search:\n";
        foreach my $db (@{$dbhash{$db_alias}})
        {
            print "\t$db\n"
        }
    }
    exit;
}

sub trap
{
    my ($db_alias, $command, $message) = @_;

    die "Search of $db_alias failed: $message when running: $command";
}


