#!/usr/local/bin/perl5.6.0 -w

=head1 NAME

embl2webtree

=head1 SYNOPSIS

 embl2webtree filename

=head1 DESCRIPTION

This script takes an annotated EMBL file which includes /class
qualifiers and creates a tree of web pages based on the functional
classification. Two things are required; the EMBL file and a classes
data file which should contain a tab-delimited list of a class name
and its description, one class per line.

 e.g. 1       Small molecule metabolism
      1.A     Degradation
      1.A.1   Carbon compounds
      1.A.2   Amino acids
      1.B     Energy metabolism
      1.B.1   Glycolysis                etc.

Note that the list indentation in the final web page is dependent on
the length of the class name (1.A.1 is a subclass of 1.A etc.)  so if
you have a class hierarchy based on some other system, you will have
to hack the script (read the HTML::Template manpage first and drink
plenty of coffee). The <ul> </ul> nesting isn't perfect yet.

You will need to edit the config at the top of the script and edit the
index.tmpl to reflect the title you require on the top page. If you
want to change the colours and formatting etc. then edit the templates
(index.tmpl, class.tmpl and cds.tmpl) as if they were HTML. The pages
produced use the Sanger header/footer CGIs. You can edit the templates
to change this too. The extra layer of HTML::Template keeps the script
and HTML (mostly - spot the hack!) separate so that you can do this.

If you choose to set a cross-reference to another CDS you will need to
supply a value for the $cds_xref variable. This should be the name of
a qualifer in the CDS containing information about the target of the
xref e.g. /orthologue="other_cds". The $xref_base variable should
contain the base of the URL to be linked to
(e.g. "http://www.sanger.ac.uk/Projects/C_jejuni") and the
$xref_suffix should contain the HTML file suffix of the target
(e.g. ".shtml"). The get_xref subroutine should then be edited to
construct a valid URL from the base, qualifier value and suffix and
return it along with the URL content text. These data are passed to
the HTML::Template for the CDS as <TMPL_VAR NAME=XREF_URL> and
<TMPL_VAR NAME=XREF_NAME> respectively.

When you run the script (assuming the example settings below) you will
 get:

 ./Cj_gene_list_hierarchical.shtml

 ./functional_classes/1.shtml
 ./functional_classes/1.A.shtml
 ./functional_classes/1.A.1.shtml etc.

 ./CDS/Cj0001.shtml
 ./CDS/Cj0002.shtml
 ./CDS/Cj0003.shtml etc.

=head1 AUTHOR

Keith James (kdj@sanger.ac.uk)

=head1 APPENDIX

=head2 Notes

=cut

use strict;
use Bio::PSU::SeqFactory;
use HTML::Template;

###
### User-configurable variables
###

# Location of the HTML templates
my $cds_template_file      = "html_templates/cds.tmpl";
my $class_template_file    = "html_templates/class.tmpl";
my $pfam_template_file     = "html_templates/pfam.tmpl";
my $prosite_template_file  = "html_templates/prosite.tmpl";
my $index_template_file    = "html_templates/index.tmpl";

my $index_pfam_file        = "html_templates/index_pfam.tmpl";
my $index_prosite_file     = "html_templates/index_prosite.tmpl";

# Location of the functional class details
my $class_file             = "functional_classes/classes.dat";

# Name of the root index files
my $index_filename         = "Cj_gene_list_hierarchical";
my $index_pfam_filename    = "index_pfam";
my $index_prosite_filename = "index_prosite";

# Suffix to be used for all HTML output files
my $suffix                 = "shtml";

# Regular expression to match the systematic gene naming scheme
# e.g. 'Cj[0-9]{4}c?' or 'NM[0-9]{4}'
my $sysgene_patt = 'Cj[0-9]{4}c?';

my $cds_xref     = undef;
my $xref_base    = "";
my $xref_suffix  = "shtml";

###
### End of user-configurable variables
###

my $wrap  = 10;
my $usage = "Usage: embl2webtree filename\n";

my (@entries, %html_templates, %classes, %header_classes);

###
### Open and read gene classification
###

open(IN, "<$class_file") or die "Unable to open class data file $class_file: $!\n";
while (<IN>)
{
    my ($class, $description) = split("\t", $_);
    $classes{$class} = $description;
}
close(IN);

###
### Process the EMBL files
###

my @files = @ARGV;
unless (@files) {die $usage};

foreach my $file (@files)
{
    my @new_entries = read_db($file);
    push(@entries, @new_entries);
}

###
### Process the EMBL entries
###

my (@cds, @pfam, @prosite, %pfam_cds, %prosite_cds);

foreach my $entry (@entries)
{
    foreach my $feature ($entry->features)
    {
    CASE:
	{
	    if ($feature->key eq 'CDS')
	    {
		push(@cds, $feature);
		last CASE;
	    }
	    if ($feature->key eq 'misc_feature')
	    {
		if (grep /^Pfam match/, $feature->qvalues('^note$'))
		{
		    push(@pfam, $feature)
		}
		elsif (grep /^PS\d{5}/, $feature->qvalues('^note$'))
		{
		    push(@prosite, $feature)
		}
		last CASE;
	    }
	    my $nothing = 1;
	}
    }
}

###
### Process each CDS
###

foreach my $cds (@cds)
{
    my ($gene, $sysgene, $prod, $note, $class, $title) = cds_details($cds, $sysgene_patt);
    print "Processing: $sysgene\n";

    my ($aa, $aa_str, $aa_len, $mwt, $xref_url, $xref_name);
    if ($cds->qexists('pseudo'))
    {
	$aa_len = "-";
	$mwt    = "-";
	$aa_str = "Not applicable";
    }
    else
    {
	$aa     = $cds->translate;
	$aa_len = $aa->length;
	$mwt    = $aa->molwt;
	$aa_str = wrap($aa->str, $wrap);
    }

    if (defined $cds_xref)
    {
	($xref_url, $xref_name) = get_xref($cds, $cds_xref, $xref_base, $xref_suffix);
    }

    ###
    ### Process the Pfam hits
    ###

    my ($pfam, $pfam_id, $pfam_text);
    foreach my $pfam_hit (@pfam)
    {
	if ($pfam_hit->overlaps($cds))
	{
	    $pfam = join("\n", $pfam_hit->qvalues('^note$'));
	    if ($pfam =~ /(PF\d{5})/)
	    {
		$pfam_id = $1;
		if (exists $pfam_cds{$pfam_id})
		{
		    push(@{$pfam_cds{$pfam_id}}, [$gene, $sysgene, $prod])
		}
		else
		{
		    $pfam_cds{$pfam_id} = [ [$gene, $sysgene, $prod] ]
		}
	    }
	    else
	    {
		die "Unable to find Pfam id for hit on CDS ", $cds->id, "\n"
	    }
	    $pfam =~ s!(PF\d{5})!<a href=\"http:\/\/www\.sanger\.ac\.uk\/cgi-bin\/Pfam\/getacc\?$1\">$1<\/a>!;
	    $pfam_text .= "$pfam<br>";
	}
    }
    $pfam = $pfam_text;
    $pfam ||= "None";
    

    ###
    ### Process the Prosite hits
    ###

    my ($prosite, $prosite_id, $prosite_text);
    foreach my $prosite_hit (@prosite)
    {
	if ($prosite_hit->overlaps($cds))
	{
	    $prosite = join("\n", $prosite_hit->qvalues('^note$'));
	    if ($prosite =~ /(PS\d{5})/)
	    {
		$prosite_id = $1;
		if (exists $prosite_cds{$prosite_id})
		{
		    push(@{$prosite_cds{$prosite_id}}, [$gene, $sysgene, $prod])
		}
		else
		{
		    $prosite_cds{$prosite_id} = [ [$gene, $sysgene, $prod] ]
		}
	    }
	    else
	    {
		die "Unable to find PS id for hit on CDS ", $cds->id, "\n"
	    }
	    $prosite=~ s!(PS\d{5})!<a href=\"http:\/\/www\.expasy\.ch\/cgi-bin\/prosite-search-ac\?$1">$1<\/a>!;
	    $prosite_text .= "$prosite<br>";
	}
    }
    $prosite = $prosite_text;
    $prosite ||= "None";

    # Create content of the gene html file
    my $cds_template = HTML::Template->new(filename => $cds_template_file);

    $cds_template->param(
			 CDS_NAME        => $title,
			 CDS_GENE        => $gene,
			 CDS_SYS         => $sysgene,
			 CDS_PRODUCT     => $prod,
			 CDS_CLASS       => $class,
			 CDS_START       => $cds->start,
			 CDS_END         => $cds->end,
			 CDS_BP          => $cds->length,
			 CDS_AA          => $aa_len,
			 CDS_MWT         => $mwt,
			 CDS_NOTE        => $note,
			 CDS_TRANSLATION => $aa_str,
			 CDS_PFAM        => $pfam,
			 CDS_PS          => $prosite
			 XREF_URL        => $xref_url,
			 XREF_NAME       => $xref_name);

    if (exists $html_templates{$class})
    {
	push(@{$html_templates{$class}}, $cds_template)
    }
    else
    {
	$html_templates{$class} = [$cds_template]
    }

    my $outfile = "CDS/$sysgene.$suffix";
    write_html($cds_template, $outfile);
}

###
### Create content of the class html file
###

my @sorted_classes = sort{$a cmp $b} (keys %classes);

foreach my $class (@sorted_classes)
{
    next unless (exists $html_templates{$class});
    my $num_members = scalar @{$html_templates{$class}};
    my @class_members;

    foreach (@{$html_templates{$class}})
    {
	my %data;
	$data{CDS_GENE}    = $_->param("CDS_GENE");
	$data{CDS_SYS}     = $_->param("CDS_SYS");
	$data{CDS_PRODUCT} = $_->param("CDS_PRODUCT");
	push(@class_members, \%data);
    }

    my $class_template = HTML::Template->new(filename => $class_template_file);

    $class_template->param
	(
	 CLASS_NAME  => $class,
	 DESCRIPTION => $classes{$class},
	 NUM_MEMBERS => $num_members,
	 MEMBERS     => \@class_members
	);

    my $outfile = "functional_classes/$class.$suffix";
    write_html($class_template, $outfile);
}

###
### Create content of the Pfam html file
###

my @sorted_pfam = sort{$a cmp $b} (keys %pfam_cds);
my @pfam_id_data;

foreach my $pfam (@sorted_pfam)
{
    my $num_members = scalar @{$pfam_cds{$pfam}};
    my %id;
    $id{PFAM_NAME}   = $pfam;
    $id{NUM_MEMBERS} = $num_members;
    push(@pfam_id_data, \%id);

    my %class_members;

    # Number of Pfam hits of this type per CDS
    my %pfam_per_cds;

    foreach (@{$pfam_cds{$pfam}})
    {
	my %data;
	$data{CDS_GENE}    = $_->[0];
	$data{CDS_SYS}     = $_->[1];
	$data{CDS_PRODUCT} = $_->[2];
	$data{PFAM_COUNT}  = $pfam_per_cds{$_->[1]};
	$class_members{$_->[1]} = \%data;
    }

    my $pfam_template = HTML::Template->new(filename => $pfam_template_file);

    $pfam_template->param(PFAM_NAME   => $pfam,
			  NUM_MEMBERS => $num_members,
			  MEMBERS     =>  [ map { $class_members{$_} }
					    sort {$a cmp $b}
					    keys %class_members ]);

    my $outfile = "pfam/$pfam.$suffix";
    write_html($pfam_template, $outfile);
}

# Create the Pfam top-level index
my $index_pfam = HTML::Template->new(filename => $index_pfam_file);
$index_pfam->param(PFAM => \@pfam_id_data);

my $index_pfam_outfile = "$index_pfam_filename.$suffix";
write_html($index_pfam, $index_pfam_outfile);

###
### Create content of the Prosite html file
###

my @sorted_prosite = sort{$a cmp $b} (keys %prosite_cds);
my @prosite_id_data;

foreach my $prosite (@sorted_prosite)
{
    my $num_members = scalar @{$prosite_cds{$prosite}};
    my %id;
    $id{PROSITE_NAME} = $prosite;
    $id{NUM_MEMBERS}  = $num_members;
    push(@prosite_id_data, \%id);

    my %class_members;

    # Number of Prosite hits of this type per CDS
    my %prosite_per_cds;

    foreach (@{$prosite_cds{$prosite}})
    {
	my %data;
	$data{CDS_GENE}    = $_->[0];
	$data{CDS_SYS}     = $_->[1];
	$data{CDS_PRODUCT} = $_->[2];
	$data{PROSITE_COUNT} = $prosite_per_cds{$_->[1]};
	$class_members{$_->[1]} = \%data;
    }

    my $prosite_template = HTML::Template->new(filename => $prosite_template_file);

    $prosite_template->param(PROSITE_NAME => $prosite,
			     NUM_MEMBERS  => $num_members,
			     MEMBERS      => [ map { $class_members{$_} }
					       sort {$a cmp $b}
					       keys %class_members ]);

    my $outfile = "prosite/$prosite.$suffix";
    write_html($prosite_template, $outfile);
}

# Create the Prosite top-level index
my $index_prosite = HTML::Template->new(filename => $index_prosite_file);
$index_prosite->param(PROSITE => \@prosite_id_data);

my $index_prosite_outfile = "$index_prosite_filename.$suffix";
write_html($index_prosite, $index_prosite_outfile);

###
### Create the content of the top (index) html file
###

my @class_data;
my $indent = 0;

foreach my $class (@sorted_classes)
{
    my %data;
    $data{CLASS}       = $class;
    $data{DESCRIPTION} = $classes{$class};

    if (exists $html_templates{$class})
    {
	$data{MEMBERS_EXIST} = 1;
	my $num_members = $#{$html_templates{$class}} + 1;
	$data{NUM_MEMBERS} = $num_members;
    }
    else
    {
	$data{MEMBERS_EXIST} = 0
    }

    my @levels = split(/\./, $class);
    my $depth = $#levels + 1;

    if ($depth == $indent)
    {
	$data{INDENTS}  = 0;
	$data{OUTDENTS} = 0;
    }
    else
    {
	if ($depth > $indent)
	{
	    $data{INDENTS}   = 1;
	    $data{OUTDENTS}  = 0;
	    $data{OPEN_LIST} = "<ul>\n" x ($depth - $indent);
	    $indent = $depth;
	}
	if ($depth < $indent)
	{
	    $data{INDENTS}    = 0;
	    $data{OUTDENTS}   = 1;
	    $data{CLOSE_LIST} = "</ul>\n" x ($indent - $depth);
	    $indent = $depth;
	}
    }
    push(@class_data, \%data);
}

my $index_template = HTML::Template->new(filename => $index_template_file);
$index_template->param(CLASSES => \@class_data);

my $outfile = "$index_filename.$suffix";
write_html($index_template, $outfile);


###
### Subroutines
###

=head2 read_db

 Title   : read_db
 Usage   : read_db(filename, format)
 Function: Opens a new Seqfactory object for reading
 Example : read_db($file, 'embl')
 Returns : List of Bio::PSU::Seq objects
 Args    : Filename string, format string

=cut

sub read_db
{
    my ($db_file) = @_;
    my @db_list;

    my $db_in = Bio::PSU::SeqFactory->make(-file => $db_file);

    while (my $seq = $db_in->next_seq)
    {
	push(@db_list, $seq)
    }
    return @db_list;
}

=head2 cds_details

 Title   : cds_details
 Usage   : cds_details(feature, systematic gene regexp)
 Function: Gets a list of qualifier values from a feature object
 Example : cds_details($feature, $sysgene_patt)
 Returns : List of strings, see code for details and order
 Args    : feature object, regexp (scalar)

=cut

sub cds_details
{
    my ($cds, $sysgene_patt) = @_;
    my (@allgene, @sysgene, @gene, @product, @note, @class);
    my ($gene_str, $sysgene_str, $prod_str, $note_str, $class_str, $title_str);

    my $null_gene_str = "-----";
    my $null_prod_str = "undefined product";

    # A gene may have 2 names e.g. ribA and Cj1218c.  Here $gene would
    # be "ribA" and $sysgene "Cj1218c" You don't know in which order
    # they will turn up and $sysgene is often missing altogether. This
    # is what the sysgene regexp is for at the top of the file. It
    # lets you know if you have found the systematic name and if it's
    # missing at the end you can try to recover it from the note
    # field.

    @product = $cds->qvalues('^product$');
    @allgene = $cds->qvalues('^gene$');
    @class   = $cds->qvalues('^class$');
    @note    = $cds->qvalues('^note$');

    @sysgene = grep  /$sysgene_patt/, @allgene;
    @gene    = grep !/$sysgene_patt/, @allgene;

    unless (@sysgene)
    {
	if ($note[0] =~ /^\s*($sysgene_patt)/)
	{
	    my $sysgene = $1;
	    push(@sysgene, $sysgene)
	}
	else
	{
	    die "Unable to recover systematic gene name at: ", $cds->start, "..", $cds->end, "\n"
	}
    }

    unless (@class)
    {
	die "Unable to recover class information at: ", $cds->start, "..", $cds->end, "\n"
    }

    $sysgene_str = shift @sysgene;
    $class_str   = shift @class;

    if (@gene)
    {
	$gene_str = join(', ', @gene);
	$title_str = $gene_str;
    }
    else
    {
	$gene_str  = $null_gene_str;
	$title_str = $sysgene_str;
    }

    $prod_str = join(', ', @product) if @product;
    $prod_str ||= $null_prod_str;

    $note_str = join(', ', @note) if @note;

    return ($gene_str, $sysgene_str, $prod_str, $note_str, $class_str, $title_str);
}

=head2 get_xref

 Title   : get_xref
 Usage   : get_xref(Bio::PSU::Feature, qualifer, URL base, URL suffix)
 Function: Creates a cross reference URL for a CDS using the contents of
         : the qualifier specified by $cds_xref
 Example : get_xref($cds, $qualifier, $xref_base, $xref_suffix)
 Returns : List (String (URL), String (URL name))
 Args    : Bio::PSU::Feature, string, string, string

=cut

sub get_xref
{
    my ($cds, $cds_xref, $xref_base, $xref_suffix) = @_;

    $cds_xref = $cds->$cds_xref;
    return unless defined $cds_xref;

    return ("$xref_base/$cds_xref.$xref_suffix", $cds_xref);
}

=head2 wrap

 Title   : wrap
 Usage   : wrap(string, line length)
 Function: Wraps a string into lines of a specified length
 Example : wrap($trans, 10)
 Returns : Nothing
 Args    : string, integer

=cut

sub wrap
{
    my ($seq, $wrap) = @_;
    my $wrapped;

    for (my $i = 0; $i <= length($seq); $i += $wrap)
    {
	$wrapped .= substr($seq, $i, $wrap) . "\n";
    }
    return $wrapped;
}

=head2 write_html

 Title   : write_html
 Usage   : write_html(HTML template, file name)
 Function: Writes the contents of an HTML::Template object to a file as HTML
 Example : write_html($index, $indexfile)
 Returns : Nothing
 Args    : HTML::Template object, file name

=cut

sub write_html
{
    my ($template, $outfile) = @_;

    open(OUT, ">$outfile") or die "Unable to open $outfile for writing: $!\n";
    print OUT $template->output();
    close(OUT);
}





