#!/nfs/newlocal/bin/perl5.6.0 -w

use strict;

use Cwd;
##########################################

# BioPerl module for GFF Parsing
use Bio::Tools::GFF;

# PSU modules for EMBL writing
use Bio::PSU::SeqFactory;
use Bio::PSU::Seq;
use Bio::PSU::Feature;

=head1 NAME

exonerate2art.pl 

=head1 SYNOPSIS

  exonerate2art.pl [gff file]

=head1 DESCRIPTION

  This script converts the GFF exonerate output into an EMBL file.
  The output file name is the name of the input file plus the suffix ".tab"
  The output file is generated in such a way that the results can be displayed into the sequence viewer software, Artemis.

  @author Arnaud Kerhornou - axk@sanger.ac.uk

=cut

=head1 ALGORITHM

=cut

$|=1;

my $curdir = cwd;
$curdir =~ s/\/tmp_mnt//g;

my $gff_file = $ARGV[0] or die "specify the name of a gff file\n";
my $taboutput = "$gff_file.tab";
open GFF, "$curdir/$gff_file";
my $gffio = Bio::Tools::GFF->new(-fh => \*GFF, -gff_version => 2);
my $new_tab_seq_obj = Bio::PSU::Seq->new (
					  -type => "dna"
					 );

# to know when we are parsing information about a new gene
# => close the current CDS feature and open a new one
my $new_gene = 0;

my $strand = 0;
my $score  = -1;
my $percent_id = -1;
my $cdna_seq_name = "";

my @new_features = ();
my @ranges = ();

while (<GFF>) {
  
  next if ($_ =~ /^#/);
	   
  my $feature = $_;

  # print STDERR "feature line before gene matching: $feature\n";

  if ($feature =~ /gene\t/) {
    
    # if a new gene has been parsed previously
    # copy the new CDS feature into the new tab file

    if ($new_gene == 1) {
 
      # the CDS feature

      my $embl_feature = Bio::PSU::Feature->new(
						-key    => 'CDS',
						-ranges => [@ranges]
					       );
      $embl_feature->qadd ('note', "cDNA sequence: $cdna_seq_name");
      $embl_feature->qadd ('note', "percent_id: $percent_id - gene score: $score");
      $embl_feature->qadd ('percent_id', $percent_id);
      $embl_feature->qadd ('score', $score);
      push (@new_features, $embl_feature);

      # reinit @ranges array
      @ranges = ();
    }

    $new_gene  = 1;

    # print STDERR "feature line after gene matching: $feature\n";
    $feature =~ /(\w+)\t+(\S+)\t+(\w+)\t+(\d+)\t+(\d+)\t+(\S+)\t+(\+|-)\t+(.+)/;

    $cdna_seq_name = $1;
    $score         = $6;
    $strand        = 0;
   
    if ($7 =~ /\+/) {
      $strand = 1;
    }
    elsif ($7 =~ /-/) {
      $strand = -1;
    }
    
    $feature =~ /.+percent_id\t+(.+)/;
    
    $percent_id = $1;
    
    # print STDERR "%: $percent_id\n";
    
    next;
  }

  # print STDERR "feature line before exon matching: $feature\n";

  if ($feature =~ /exonerate\t+exon/) {

    # print STDERR "feature line : $feature\n";
    $feature =~ /.+\t+(\d+)\t+(\d+)\t+.+/;

    my $start = $1;
    my $end   = $2;

    # print STDERR "start, end, strand: $start, $end, $strand\n";

    my $range = Bio::PSU::Range->new(-start       => $start,
				     -end         => $end,
				     -strand      => $strand);
    push (@ranges, $range);
  }
}
$gffio->close();
close GFF;

if ($new_gene == 1) {

  # the CDS feature

  my $embl_feature = Bio::PSU::Feature->new(
					    -key    => 'CDS',
					    -ranges => [@ranges]
					   );
  $embl_feature->qadd ('note', "cDNA sequence: $cdna_seq_name");
  $embl_feature->qadd ('note', "percent_id: $percent_id - gene score: $score");
  $embl_feature->qadd ('percent_id', $percent_id);
  $embl_feature->qadd ('score', $score);
  
  push (@new_features, $embl_feature);
}

# done => write the generated features into an EMBL file

$new_tab_seq_obj->features (@new_features);
  
# copy this seq obj into a new file
  
my $taboutput_path = $curdir . '/' . $taboutput;
my $new_tab_obj = Bio::PSU::SeqFactory->make(-file   => ">$taboutput_path",
					     -format => 'embl');
$new_tab_obj->write_seq($new_tab_seq_obj);  
  
#############################################
# The End
#############################################
