#!/usr/local/bin/perl5.6.0 -w

=head1 NAME

frep

=head1 SYNOPSIS

frep -k key -q qualifier -g regex file

Additionally, use arguments -f to print features, -s to print sequence
(with -u and -d to print extra residues upstream and downstream, -m
to specify sequence format)

=head1 DESCRIPTION

frep is like grep, but allows you to grep the values within the
qualifiers of EMBL features and then print the matching features or
their sequence.

Usage: frep [-gkq <arg>] [-fsvn] [-ud <int>] [-m <arg>] file
    -k regular expression to match feature key
    -q regular expression to match qualifier name
    -g regular expression to grep qualifier value
    -n additionally match qualifiers which have no value
    -v show all features except those which match

    -f output the feature itself
    -s output the feature sequence
    -u include this number of bases upstream for sequence
    -d include this number of bases downstream for sequence
    -m output format for sequence (EMBL, Fasta)

The arguments are optional (using no arguments all features will
match).

The default is to output an id and description for matching features.
If the -f switch is used then the whole EMBL formatted feature will be
output. If the -s switch is used the feature sequence will be output.
The -u, -d and -m switches only apply if the -s switch is used as they
specify sequence output modifiers.

The -n switch allows inclusion of qualifiers which have no value set
e.g. /pseudo. If this option is on, these features will always match
, regardless of the -g switch.

=head1 AUTHOR

Keith James <kdj@sanger.ac.uk>

=cut

use strict;
use Getopt::Std;
use Bio::PSU::SeqFactory;

my $fkey = '.';
my $qkey = '.';
my $gkey = '.';

my $upstream = 0;
my $dnstream = 0;

my $fout   = 0;
my $sout   = 0;
my $nullok = 0;
my $allok  = 0;
my $invert = 0;
my $format = 'fasta';

my %opts;

my $usage = "Usage: frep [-gkq <arg>] [-fsvn] [-ud <int>] [-m <arg>] file
\t-k feature key
\t-q regular expression to match qualifier
\t-g regular expression to grep in qualifier
\t-n additionally match qualifiers which have no value
\t-a match all features with that key, even if they have no qualifiers
\t-v show all features except those which match

\t-f output the feature itself
\t-s output the feature sequence
\t-u include this number of bases upstream for sequence
\t-d include this number of bases downstream for sequence
\t-m output format for sequence (EMBL, Fasta)

The default is to output an id and description for matching features.
If the -f switch is used then the whole EMBL formatted feature will be
output. If the -s switch is used the feature sequence will be output.
The -u, -d and -m switches only apply if the -s switch is used as they
specify sequence output modifiers

The -n switch allows inclusion of qualifiers which have no value set
e.g. /pseudo. If this option is on, these features will always match
, regardless of the -g switch. With the -a switch all features with
the requested key(s) with match, even those which have no qualifiers
at all
";

getopts('d:g:k:m:q:u:agsfhvn', \%opts);

defined $opts{"k"} and $fkey     = $opts{"k"};
defined $opts{"q"} and $qkey     = $opts{"q"};
defined $opts{"g"} and $gkey     = $opts{"g"};
defined $opts{"h"} and die $usage;

defined $opts{"v"} and $invert++;
defined $opts{"n"} and $nullok++;
defined $opts{"a"} and $allok++;
defined $opts{"f"} and $fout++;
defined $opts{"s"} and $sout++;

defined $opts{"m"} and $format   = $opts{"m"};
defined $opts{"u"} and $upstream = $opts{"u"};
defined $opts{"d"} and $dnstream = $opts{"d"};

unless ($format =~ /fasta|embl/i) { die "Invalid output format: $format\n" }

if ($fout and $sout)
{
    die "Switches -f and -s are mutually exclusive; please choose one or the other\n"
}

my @files = @ARGV;
unless (@files) {die $usage};

my $fth  = Bio::PSU::IO::FTHandler->new;
my $seqo = Bio::PSU::SeqFactory->make(-fh     => \*STDOUT,
									  -format => $format);

FILE: foreach my $file (@files)
{
    unless (-e $file)
    {
		warn "File $file does not exist\n";
		next FILE;
    }

    unless (-r $file)
    {
		warn "File $file is not readable\n";
		next FILE;
    }

    my $seqi = Bio::PSU::SeqFactory->make(-file => $file);
    my $uniquifier = 0;

 SEQ: while (my $seq = $seqi->next_seq)
    {
    FEAT: foreach my $feature ($seq->features)
        {
			next FEAT unless $feature->key =~ /$fkey/;

			my $hits = 0;

			if ($allok)
			{
				$hits++
			}
			else
			{
				$hits += grep /$gkey/, $feature->qvalues($qkey);
			}

			if ($nullok and ! $hits)
			{
				$hits++ if $feature->qexists($qkey)
			}

			if ($invert)
			{
				next FEAT if $hits;
				$hits++;
			}

			next FEAT unless $hits;

			if ($fout)
			{
				print $fth->make_feature_block($feature);
				next FEAT;
			}

			$uniquifier++;

			my $id = $feature->label;
			$id  ||= $feature->gene;
			$id  ||= $feature->key . "_". $uniquifier;

			my $start = $feature->start;
			my $end   = $feature->end;

			my $desc = $feature->product;
			$desc  ||= $feature->note;
			$desc  ||= "";

			$desc .= " [" . $start . ".." . $end . "] strand " . $feature->strand;

			if ($sout)
			{
				unless (defined $feature->str)
				{
					warn "Feature table of entry ", $seq->id, " has no sequence attached\n";
					last FEAT;
				}

				if ($feature->strand == -1)
				{
					($upstream, $dnstream) = ($dnstream, $upstream)
				}

				$start = $start - $upstream;
				$end   = $end   + $dnstream;

				if ($start < 1) { $start = 1 }
				if ($end > $seq->length) { $end = $seq->length }

				$desc .= " $upstream US, $dnstream DS";

				my $fseq = $seq->subseq($start, $end);
				$fseq    = $fseq->revcom if $feature->strand == -1;

				$fseq->id("$id.$uniquifier");
				$fseq->desc($desc);
				$fseq->type($feature->type);

				$seqo->write_seq($fseq);

				next FEAT;
			}
			print "$id $desc\n";
		}
    }
}
