#!/usr/local/bin/perl -w

# $Header: /nfs/pathsoft/repository/psu/genexec/perl/src/mark_introns.pl,v 1.1 2000/08/15 15:49:13 kmr Exp $

# purpose: This script will generate a feature table file with one feature for
# purpose: each branch and each donor.

use strict;

use Embl;
use EMBL::Parser;

sub usage
{
  die <<EOF;
usage $0: -h (for help) 
   or $0 [-dicty] embl_file ...
EOF
}

if (!@ARGV || $ARGV[0] eq "-h") {
  usage ();
}

my @branch_patterns = ('ytray','cttac|ctaaa|ataac|ctcac|gtaac');

if ($ARGV[0] eq '-dicty') {
  @branch_patterns = ('ytray','cttac|ctaaa|ataac|ctcac|gtaac');
  shift;
} else {
  warn "defaulting to pombe branch consensus\n";
}


# now substitute a or g for r and c or t for y
for my $branch_pattern (@branch_patterns) {
  $branch_pattern =~ s/r/[ag]/g;
  $branch_pattern =~ s/y/[ct]/g;

  print STDERR "searching for $branch_pattern\n";
}

# read in some embl files from a stream
my $set = Read Embl(\*ARGV);  # read all of the embl entries from that stream.


# get an array of all of the EMBL::Entry objects within $set
my @entries = $set->entries();
while (my $entry = pop @entries) {
  my $ft = $entry->FT ();

  my @features = $ft->features ();
  
  my $sequence = $entry->SQ ();

  my $length = $sequence->length ();

  my $sequence_string = $sequence->seq;

  my @introns = ();

#  print "sequence length = $length\n";

  # loop over the features in the current entry
  FEATURE: for my $f (@features) {
    my %feature_hash = %$f;

    if ($feature_hash{key} ne "CDS") {
      # go straight to the next feature - we are only interested in
      # CDS features
      next;
    }

#       print "feature: $$f{key}   $$f{location}\n";

    my @this_feature_introns = ();

    my $location = $feature_hash{location};

    my $complement_join_flag = 0;

    if ($location =~ m/complement\(join\(/) {
      $complement_join_flag = 1;
    }
    
    my $complement_flag = 0;

    if ($location =~ s/complement\(([^\)]+)\)/$1/g) {
      $complement_flag = 1;
    }

    if ($location =~ s/join\(([^\)]*)\)/$1/g) {
      
      my @join_bits = split (',',$location);

      if ($complement_flag && ! $complement_join_flag) {
        @join_bits = reverse @join_bits;
      }

      # for each pair of exons ranges look for branches and donors in the
      # intron

      for (my $i = 0 ; $i < @join_bits - 1 ; ++$i) {
        my $left_exon = $join_bits[$i];
        my $right_exon = $join_bits[$i+1];

        my $start_of_intron;
        my $end_of_intron;

        if ($left_exon =~ 
            m/(?:<?
               (\d+)
               \.\.
               >?)?
            (\d+)/x) {

          $start_of_intron = $2+1;

          if ($right_exon =~ 
              m/<?(\d+)
              (?:\.\.
               >?
               (\d+))?/x) {

            $end_of_intron = $1-1;

            push @this_feature_introns, [$start_of_intron, $end_of_intron,
                                         $complement_flag];

          } else {
            # the right hand side intron doesn't look like a range
            warn "Failed to understand this location: $location";
            next;
          }
        } else {
          # the left hand side intron doesn't look like a range
          warn "Failed to understand this location: $location";
          next;
        }
      }

    } else {

      # this isn't a spliced gene
    }

#      if ($join_complement_flag) {
#        push @introns, reverse @this_feature_introns;
#      } else {
      push @introns, @this_feature_introns;
#      }
  }

  # loop over the introns

  for my $intron (@introns) {
    my $start_of_intron = $intron->[0];
    my $end_of_intron = $intron->[1];
    my $complement_flag = $intron->[2];

#    print "$start_of_intron $end_of_intron\n";

    my $subseq_str = $sequence->subseq ($start_of_intron - 1, 
                                        $end_of_intron);

    my $intron_seq = new Sequence ();

    $intron_seq->seq ($subseq_str);

    if ($complement_flag) {
      $intron_seq->revcomp ();
    }

    my $intron_string = $intron_seq->seq;

    for my $branch_pattern (@branch_patterns) {
#print "$intron_string $branch_pattern\n";

      if ($intron_string =~ m/(?:.*)((?:$branch_pattern).*?...ag$)/i) {
        if (length $1 > 30) {
#          print ("too long: ", (length $1), "$1\n");

          # this match is too long
          next;
        }

        my $feature_start;
        my $feature_end;

        if ($complement_flag) {
          $feature_start = $start_of_intron + (length $1) - 1;
          $feature_end = $start_of_intron;
        } else {
          $feature_start = $end_of_intron - (length $1) + 1;
          $feature_end = $end_of_intron;
        }

        my $location_string = "$feature_start..$feature_end";

        if ($complement_flag) {
          $location_string = "complement(" . $location_string . ")";
        }

        print <<EOF;
FT   misc_feature    $location_string
FT                   /note="$1, splice branch and acceptor"
FT                   /colour=6
EOF
        last;
      } else {
#        print ("a: ", (substr $intron_string, 0, 50), " $start_of_intron $end_of_intron\n");
#        print ("a: ", (substr $intron_string, -50, 50),"\n");
      }
    }
      
    if ($intron_string =~ m/^(g[tc]....)/i) {
      my $feature_start;
      my $feature_end;

      if ($complement_flag) {
        $feature_start = $end_of_intron - (length $1) + 1;
        $feature_end = $end_of_intron;
      } else {
        $feature_start = $start_of_intron;
        $feature_end = 
          $start_of_intron + (length $1) - 1;
      }

      my $location_string = "$feature_start..$feature_end";

      if ($complement_flag) {
        $location_string = "complement(" . $location_string . ")";
      }

      print <<EOF;
FT   misc_feature    $location_string
FT                   /note="$1, splice donor sequence"
FT                   /colour=6
EOF

    } else {
#      print ((substr $intron_string, 0, 50), " $start_of_intron $end_of_intron\n");
#      print ((substr $intron_string, -50, 50),"\n");
    }
  }
}

