#!/usr/local/bin/perl -w
#
# $Header: /nfs/pathsoft/repository/psu/genexec/perl/src/mark_readpairs.pl,v 1.4 2000/02/21 09:37:12 kdj Exp $
#
# name:    mark_readpairs
# author:  Keith James  kdj@sanger.ac.uk
#
# purpose: Given a tab file of blastn hits to a shotgun database this 
# purpose: will colour read-pairs with a different colour for each 
# purpose: member of the pair. Unpaired reads are left uncoloured and
# purpose: may be filtered out if desired

use strict;
use Getopt::Std;
use Embl;
use Feature;

my %opts;
my ($base, $pcol_val, $qcol_val, $all) = ("", 2, 3, "");

my @entries;
my %p_reads;
my %q_reads;

getopts('b:p:q:a', \%opts);

defined $opts{"b"} and $base = $opts{"b"};
defined $opts{"p"} and $pcol_val = $opts{"p"};
defined $opts{"q"} and $pcol_val = $opts{"q"};
defined $opts{"a"} and $all++;

my $usage = "Usage: mark_readpairs -[bpqa] blastn_file.tab

\t-b base pattern (e.g. lep for reads named lep319d05.p1c/lep319d05.q1c)
\t-p Artemis colour code for p-reads (default 2 =red)
\t-q Artemis colour code for q-reads (default 3 -green)
\t-a print all blastn hits, including unpaired ones (default is to filter)
";

my $file = shift;

unless (defined $file) { die $usage }

###
### open and read the Embl file
###

open(FH, "$file") or die "Unable to open $file: $!\n";
my $newset = Embl->Read(\*FH);
push(@entries, $newset->entries());
close(FH);

my $f_table = $entries[0]->FT();
my @features = $f_table->features();

foreach my $feature (@features)
{
    $feature-> key() eq "BLASTN_HIT" and do
    {
	my @qualifiers = $feature->qualifiers();

	foreach my $qualifier (@qualifiers)
	{
	    $qualifier->{ "name" } eq "note" and do
	    {
		# read pairs are either foo.p1c & foo.q1c or foo.p1t & foo.q1t
		my $val = $qualifier->{ "value" };
		next unless $val =~ /(\b$base\w+\.([pq])1[ct])/;

		# e.g. for foo.p1c: read_name = foo.p1c, read_type = p
		my $read_name = $1;
		my $read_type = $2;

		# store list of 'p' features in a hash bucket
		if ($read_type =~ /p/)
		{
		    if (exists $p_reads{ $read_name })
		    {
			push(@{ $p_reads{ $read_name } }, $feature);
		    }
		    else
		    {
			$p_reads{ $read_name } = [ $feature ];
		    }
		  }

		# store list of 'q' features in a hash bucket
		if ($read_type =~ /q/)
		{
		    if (exists $q_reads{ $read_name } )
		    {
			push(@{ $q_reads{ $read_name } }, $feature);
		    }
		    else
		    {
			$q_reads{ $read_name } = [ $feature ];
		    }
		}
	    };
	}
    }
}

# if there is a matching 'q' read for the 'p' read, add a different colour
# qualifier to each feature in each list and print them all
foreach my $p_read (keys %p_reads)
{
    $p_read =~ /(.*)([pq]1[ct])/;
    my ($prefix, $p_suffix) = ($1, $2);
    my $q_suffix = $p_suffix;
    $q_suffix =~ tr/p/q/;

    if (exists $q_reads{ "$prefix$q_suffix" })
    {
	my $p_colour = Feature::Qualifier->fromNameValue('colour', $pcol_val);
	my $q_colour = Feature::Qualifier->fromNameValue('colour', $qcol_val);

	foreach (@{ $p_reads{ "$prefix$p_suffix" } })
	{
	    $_->addQualifiers($p_colour);
	    $_->toEMBLStream(\*STDOUT);
	}

	foreach (@{ $q_reads{ "$prefix$q_suffix" } })
	{
	    $_->addQualifiers($q_colour);
	    $_->toEMBLStream(\*STDOUT);
	}

	delete $p_reads{ "$prefix$p_suffix" };
	delete $q_reads{ "$prefix$q_suffix" };
      }
}


if ($all)
{
    foreach (keys %p_reads)
    {
	foreach (@{ $p_reads{ "$_" } })
	{
	    $_->toEMBLStream(\*STDOUT);
	}
      }

    foreach (keys %q_reads)
    {
	foreach (@{ $q_reads{ "$_" } })
	{
	    $_->toEMBLStream(\*STDOUT);
	}
    }
}
