#!/usr/local/bin/perl5.6.0 -w

# $Header: /nfs/pathsoft/repository/psu/genexec/perl/src/psu_to_go.pl,v 1.1 2002/02/13 17:07:57 kmr Exp $

# purpose: This script reads the /GO_component, /GO_process and /GO_function
# purpose: qualifiers from the file passed on the command line and writes a
# purpose: Gene Ontology consortium format gene association file to STDOUT


use strict;

use Bio::PSU::SeqFactory;
use Bio::PSU::IO::BufferFH;

sub get_range_string
{
  my ($feature) = shift;
  my @ranges = map {
    $_->{start} . ".." . $_->{end}
  } $feature->ranges ();

  join ",", @ranges;
}

if (@ARGV != 1) {
  die <<EOF;
usage $0 tab_file

 This script reads the /GO_component, /GO_process and /GO_function
 qualifiers from the file passed on the command line and writes a
 Gene Ontology consortium format gene association file to STDOUT
EOF
}

# try to read it as a tab file
my $seqfactory =
  Bio::PSU::SeqFactory->make(-file   => $ARGV[0],
                             -format => "EMBL");

my $seq = $seqfactory->next_seq;

my $time;

{
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime;

  $year += 1900;

  $time = sprintf "%s%2.2d%2.2d", $year, $mon, $mday;

  print "$time\n";
}

if (defined $seq) {
  my @features = ();

 FEATURE:
  for my $feature ($seq->features) {
    next unless $feature->key () eq "CDS";

    my @genes = ($feature->gene (), $feature->label());

    if (!@genes) {
      warn "no /gene at: ", get_range_string($feature), "\n";
      next FEATURE;
    }

    my ($pfc_gene_name, $mal_gene_name);

    for my $gene (map {split /\s*,\s*/, $_} @genes) {
      if ($gene =~ /(PF\S+[wc])/) {
        $pfc_gene_name = $1;
      } else {
        if ($gene =~ /(MAL\S+\.\d+)/) {
          $mal_gene_name = $1;
        } else {
          warn "can't fathom this /gene name: $gene\n";
        }
      }
    }

    if (!defined $pfc_gene_name or !defined $mal_gene_name) {
      warn "missing a MAL... gene name or a FPC... gene name at: ",
        get_range_string ($feature), "\n";
      next FEATURE;
    }


    my @GO_component = ();

    if ($feature->qexists ("GO_component")) {
      @GO_component = $feature->qvalues ("GO_component");
    }

    my @GO_function = ();

    if ($feature->qexists ("GO_function")) {
      @GO_function = $feature->qvalues ("GO_function")
    }

    my @GO_process = ();

    if ($feature->qexists ("GO_process")) {
      @GO_process = $feature->qvalues ("GO_process");
    }

    my @all_go_qualifiers = ();

    push @all_go_qualifiers, map { "component; $_" } @GO_component;
    push @all_go_qualifiers, map { "function; $_" } @GO_function;
    push @all_go_qualifiers, map { "process; $_" } @GO_process;

    for my $go_qualifier (@all_go_qualifiers) {
      $go_qualifier =~ s/^\s*(.*)\s*$/$1/;

      my ($ontology, $go_id, $evidence_code, $ev_code_db_ids, $the_rest) =
        split /\s*;\s*/, $go_qualifier, 5;

      if (!defined $go_id) {
        warn "no go id in /GO_$ontology qualifier for $mal_gene_name\n";
        next FEATURE;
      }

      if (!defined $evidence_code) {
        warn "no evidence_code in /GO_$ontology qualifier for $mal_gene_name\n";
        next FEATURE;
      }

      my $ontology_id;

      if ($ontology =~ /^([pcf])/i) {
        $ontology_id = uc $1;
      } else {
        die "internal error";
      }

      my $source_ev_code;
      my $source_db_id;

      if (defined $the_rest && $the_rest =~ /source \((\S+)\s*;\s*(\S+)\)/) {
        $source_ev_code = $1;
        $source_db_id = $2;
      }

      my $not_flag = 0;

      if ($go_id =~ s/^NOT //) {
        $not_flag = 1;
      }

      my $go_id_description;

      $go_id =~ s/(\S*)\s*\((.*)\)/$1/;

      $go_id_description = $2;

      if (defined $ev_code_db_ids) {
        $ev_code_db_ids =~ s/\s*\(/|/;
        $ev_code_db_ids =~ s/\)//;

        $ev_code_db_ids =~ s/SWALL:/SP:/g;
      } else {
        $ev_code_db_ids = "";
      }

      my @products = $feature->product ();

      if (!@products) {
        die get_range_string ($feature);
      }

      my $product = join "|", @products;

      print "SANGER\t$mal_gene_name\t$pfc_gene_name\t\t$go_id\tPMID:unpub\t$evidence_code\t$ev_code_db_ids\t$ontology_id\t$product\t\tgene\ttaxon:5833\t$time\n";
    }
  }
} else {
  die "End of file while reading tab file\n";
}
